#include "papu_internals.hh"
#include "papu_decayunit.hh"

void decayunit::write(fBYTE ind, fBYTE value)
{
    if(ind == 0)
    {
        decayrate = value & 15;
        decaymode = (value>>4) & 3;
        #if PAPU_DECAY_DEBUG
        if(decaymode & 1)
        	fprintf(stderr, "[\33[0;1;34mvolume=%u\33[m]", decayrate);
        else if(decaymode & 2)
        	fprintf(stderr, "[\33[0;36mdrate=%u,looping\33[m]", decayrate);
        else
        	fprintf(stderr, "[\33[0;36mdrate=%u\33[m]", decayrate);
    	#endif
    }
    else if(ind == 3)
    {
        #if PAPU_DECAY_DEBUG
    	fprintf(stderr, "[\33[0;1;34mdcounter=15,was %u\33[m]", decaycounter);
    	#endif
    	
    	/* Only a write out to $4003/$4007/$400F will reset the
         * current envelope decay counter to a known state (to
         * $F, the maximum volume level) for the appropriate
         * channel's envelope decay hardware.
         */
    	decaycounter = 15;
    }
    // Decay rate is the speed it changes with.
}

unsigned decayunit::getvol() const
{
	if(decaymode & 1)
	{
		/* When the envelope decay bit (bit 4) is set(1), the
		 * current volume value (bits 0-3) is sent directly to
		 * the channel DAC.
		 */
		return decayrate; 
	}
	
	/* When the envelope decay disable bit is cleared, bits 0-3
	 * now control the envelope decay rate, and an internal 4-bit
	 * down counter (hereon the envelope decay counter) now
	 * controls the channel's volume level.
	 */    	
    return decaycounter;
}

BEGIN_HZTICKER(240, decayunit)
    if(!decaypos)
    {
        // This counter is for timing; it always runs.
        decaypos = decayrate;                
        
        // Decay counter runs always, is decay bit (bit 4) set or not.
        
        if(decaycounter > 0)
            --decaycounter;
        else if(decaymode & 2)
            decaycounter = 15;
    }
    else
        --decaypos;
END_HZTICKER
